package main

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"net/http/cookiejar"
	"time"

	"golang.org/x/net/publicsuffix"
)

type DynmapConfig struct {
	URL       string
	Name      string
	Config    *Config
	cookiejar *cookiejar.Jar
	client    *http.Client
}

func NewDynmapConfig(name, url string) *DynmapConfig {
	d := new(DynmapConfig)
	d.Name = name
	d.URL = url
	var err error
	d.cookiejar, err = cookiejar.New(&cookiejar.Options{PublicSuffixList: publicsuffix.List})
	if err != nil {
		log.Fatal(err)
	}
	return d
}

func (d *DynmapConfig) Update() error {
	if err := d.FetchConfig(); err != nil {
		return err
	}

	if err := d.FetchAllWorlds(); err != nil {
		return err
	}
	return nil
}

func (d *DynmapConfig) GetWorlds() []World {
	return d.Config.Worlds
}

func (d *DynmapConfig) FetchConfig() error {
	body := d.fetchURL(d.URL + "configuration")
	return json.Unmarshal(body, &d.Config)
}

func (d *DynmapConfig) FetchAllWorlds() error {
	var err error
	for i, v := range d.Config.Worlds {
		d.Config.Worlds[i].Status = new(WorldStatus)
		url := fmt.Sprintf("%sworld/%s/%d", d.URL, v.Name, time.Now().Unix())
		body := d.fetchURL(url)
		if jsonErr := json.Unmarshal(body, d.Config.Worlds[i].Status); jsonErr != nil {
			err = jsonErr
		}
	}
	return err
}

func (d *DynmapConfig) GetTheStatusString() string {
	ret := "~ " + d.Config.Worlds[0].Status.ParsedTime() + " ~\n"
	longest := 0
	for _, v := range d.Config.Worlds {
		if len(v.Title) > longest {
			longest = len(v.Title)
		}
	}
	for _, v := range d.Config.Worlds {
		title := v.Title
		for i := 0; len(title) < longest; i++ {
			title = title + " "
		}
		ret = fmt.Sprintf("%s[%s]\t\t\t\t%d (%d)\n", ret, title, v.Status.CurrentCount, len(v.Status.Players))
	}
	return ret
}

func (d *DynmapConfig) GetServerTime() string {
	return d.Config.Worlds[0].Status.ParsedTime()
}

func (d *DynmapConfig) GetOnlinePlayerCount() int {
	return d.Config.Worlds[0].Status.CurrentCount
}

func (d *DynmapConfig) GetPlayerList() []string {
	return d.Config.Worlds[0].Status.Players
}

func (d *DynmapConfig) fetchURL(url string) []byte {
	client := &http.Client{Jar: d.cookiejar}
	resp, err := client.Get(url)
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return []byte{}
	}
	return body
}

type Config struct {
	Worlds []World
}

type World struct {
	Sealevel    int
	Protected   bool
	Center      *Coord
	Name        string
	Title       string
	WorldHeight int
	Status      *WorldStatus
}

type WorldStatus struct {
	CurrentCount int
	Players      []string

	HasStorm     bool
	IsThundering bool
	ConfigHash   int
	ServerTime   int
}

func (w *WorldStatus) ParsedTime() string {
	hrs := ((w.ServerTime / 1000) + 6) % 24
	mins := (w.ServerTime / 1000)
	return fmt.Sprintf("%d:%02d", hrs, mins)
}

type Coord struct {
	X float64
	Y float64
	Z float64
}
